// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#if 0 // The public interface of TCPAcceptor

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * An acceptor that provides a TCPSocket for each accepted
 * connection.
 */
class TCPAcceptor : public FailableReference<TCPAcceptor>,
                    public BufferedStreamAcceptor
{
public:
    /**
     * Start accepting a connection.
     *
     * If a connection is already accepted, but not retrieved yet, it
     * returns true.
     *
     * If a new socket becomes immediately available, it returns true.
     *
     * If some error occurs, the acceptor is marked as failed.
     */
    bool accept();

    /**
     * Get the TCP socket accepted last. If no socket was accepted,
     * return 0. The TCP socket's waiter will be the given one. If
     * the server socket has no waiter, this one should be 0. If the
     * server socket has a waiter, this one should not be 0. In other
     * words, if the server socket is non-blocking, the accepted one
     * should also be non-blocking, if the server socket is blocking,
     * the accepted one should also be blocking.
     */
    TCPSocket* getSocket(Waiter* waiter,
                         size_t readingCapacity = LocalSocket::DEFAULT_CAPACITY,
                         size_t writingCapacity = LocalSocket::DEFAULT_CAPACITY);
    /**
     * Get the TCP socket accepted last. If no socket was accepted,
     * return 0. The TCP socket's waiter will be the same as that of
     * the associated server socket.
     */
    TCPSocket* getSocket(size_t readingCapacity = LocalSocket::DEFAULT_CAPACITY,
                         size_t writingCapacity = LocalSocket::DEFAULT_CAPACITY);

    /**
     * Get the last accepted socket with the given waiter, if any.
     */
    BufferedStream* getAcceptedBufferedStream(Waiter* waiter = nullptr) override;
};

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

#endif // 0, The public interface of LocalAcceptor

//------------------------------------------------------------------------------

#include "hu/varadiistvan/scpl/config.h"

//------------------------------------------------------------------------------

#if TARGET_API_POSIX

#include "_posix/TCPAcceptor.h"

//------------------------------------------------------------------------------

#elif TARGET_API_WIN32 || TARGET_API_WIN64

#include "_win32/TCPAcceptor.h"

//------------------------------------------------------------------------------

#else

#error "Not implemented for this target!"

#endif

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
